const esprima = require("esprima");
const fs = require("fs");

function analyzeCode(file) {
  const code = fs.readFileSync(file, "utf-8");
  const parsed = esprima.parseModule(code, { comment: true, range: true });

  const functions = [];
  const classes = [];

  parsed.body.forEach((node) => {
    // Regular function
    if (node.type === "FunctionDeclaration") {
      functions.push({
        name: node.id.name,
        doc: findCommentBefore(node, parsed.comments)
      });
    }

    // Arrow function (e.g. const fn = () => {})
    if (
      node.type === "VariableDeclaration"
    ) {
      node.declarations.forEach((decl) => {
        if (
          decl.init &&
          (decl.init.type === "ArrowFunctionExpression" ||
           decl.init.type === "FunctionExpression")
        ) {
          functions.push({
            name: decl.id.name,
            doc: findCommentBefore(decl, parsed.comments)
          });
        }
      });
    }

    // Class
    if (node.type === "ClassDeclaration") {
      classes.push({
        name: node.id.name,
        doc: findCommentBefore(node, parsed.comments)
      });
    }
  });

  return { functions, classes };
}

function findCommentBefore(node, comments) {
  if (!comments) return null;

  const nodeStart = node.range[0];

  const closestComment = comments
    .filter((c) => c.range[1] < nodeStart)
    .sort((a, b) => b.range[0] - a.range[0])[0];

  return closestComment ? closestComment.value.trim() : null;
}

module.exports = { analyzeCode };
